#!/bin/sh
##
##  dex-setup.sh -- Dex RDBMS Setup Utility
##

#   command line argument sanity check
prg="$0"
if [ $# -eq 0 ]; then
    echo "$prg:ERROR: invalid command line" 1>&2
    echo "$prg:USAGE: $prg install [<database-directory>]" 1>&2
    echo "$prg:USAGE: $prg uninstall" 1>&2
    exit 1
fi

#   database configuration
db_type="@db_type@"
db_dir="@l_prefix@/var/dex/db"
db_name="dex"
db_user="dex"
db_pass="dex"

#   determine RDBMS-specific details
if [ ".$db_type" = .sqlite ]; then
    db_sname=""
    db_suser="@l_rusr@"
    db_spass="@l_rgrp@"
elif [ ".$db_type" = .pgsql ]; then
    db_sname=`grep "^superuser_database" @l_prefix@/var/postgresql/db/pg_superuser.conf |\
              sed -e 's;^ *superuser_database="\(.*\)".*;\1;'`
    db_suser=`grep "^superuser_username" @l_prefix@/var/postgresql/db/pg_superuser.conf |\
              sed -e 's;^ *superuser_username="\(.*\)".*;\1;'`
    db_spass=`grep "^superuser_password" @l_prefix@/var/postgresql/db/pg_superuser.conf |\
              sed -e 's;^ *superuser_password="\(.*\)".*;\1;'`
fi

#   dispatch operation
cmd="${1:-"install"}"
shift
case "$cmd" in
    install )
        ##
        ##  create the database
        ##
        if [ $# -gt 0 ]; then
            db_dir="$1"
            shift
        fi
        if [ ".$db_type" = .sqlite ]; then
            ( umask 077
              ( echo "CREATE TABLE dummy (dummy INTEGER);"
                echo "DROP TABLE dummy;"
              ) | @l_prefix@/bin/sqlite3 $db_dir/dex.db
              chown $db_suser:$db_spass $db_dir/dex.db
              chmod 640 $db_dir/dex.db
            ) || exit $?
        elif [ ".$db_type" = .pgsql ]; then
            ( echo "CREATE ROLE $db_user LOGIN ENCRYPTED PASSWORD '$db_pass' NOCREATEDB NOCREATEROLE;"
              echo "CREATE TABLESPACE $db_name OWNER $db_user LOCATION '$db_dir';"
              echo "CREATE DATABASE $db_name WITH ENCODING 'UTF8' TEMPLATE template0 OWNER $db_user TABLESPACE $db_name;"
              echo "GRANT ALL PRIVILEGES ON DATABASE $db_name TO $db_user;"
            ) | PGPASSWORD="$db_spass" @l_prefix@/bin/psql -q -U "$db_suser" -d "$db_sname" -f-
        fi
        ;;

    uninstall )
        ##
        ##  remove the database
        ##
        if [ ".$db_type" = .sqlite ]; then
            rm -f $db_name >/dev/null 2>&1 || true
        elif [ ".$db_type" = .pgsql ]; then
            ( echo "DROP DATABASE $db_name;"
              echo "DROP TABLESPACE $db_name;"
              echo "DROP ROLE $db_user;"
            ) | PGPASSWORD="$db_spass" @l_prefix@/bin/psql -q -U "$db_suser" -d "$db_sname" -f-
        fi
        ;;
esac

